var TTUApp = {
    
    prepareForAppReady : function() {
        this.setupFunctions.setupInjectedFunctions();
    },
    
	init : function(hideNavElements) {
		this.nextButtonNames = ['next_', 'continue_'];
		this.backButtonNames = ['back_'];
		this.buttonList = [this.nextButtonNames, this.backButtonNames];
		this.pageDoneLoading = false;
        this.modalShown = false;
		
		this.setupFunctions.setupWebview();
        
        if(hideNavElements) {
            this.functions.hideElementsOnSetup();
        }
        
		return this;
	},
    
    navigationFunctions : {
        /** If possible, performs a click action on the next button **/
		navigateForward : function() {
			goForwardFunction = function(id, button) {
				if(TTUApp.functions.doesIdMatchIdKind(id, TTUApp.nextButtonNames)) {
					button.click();
					return true;
				}
				return false;
			}
		
			TTUApp.functions.runOperationForButtons(goForwardFunction);
		},

        /**If possible, performs a click action on the back button **/
		navigateBack : function() {
			goBackFunction = function(id, button) {
				if(TTUApp.functions.doesIdMatchIdKind(id, TTUApp.backButtonNames)) {
					button.click();
					return true;
				}
				return false;
			}
		
			TTUApp.functions.runOperationForButtons(goBackFunction);
		},

        /** Checks if a next button exists on the page **/
		canNavigateForward : function() {
			canNavigateForwardFunction = function(id, button) {
				return TTUApp.functions.doesIdMatchIdKind(id, TTUApp.nextButtonNames);
			}
		
			return TTUApp.functions.runOperationForButtons(canNavigateForwardFunction);
		},

        /** Checks if there is a back button on the page **/
		canNavigateBack : function() {
			canNavigateBackFunction = function(id, button) {
				return TTUApp.functions.doesIdMatchIdKind(id, TTUApp.backButtonNames);
			}
		
			return TTUApp.functions.runOperationForButtons(canNavigateBackFunction);
		}
    },
    
    setupFunctions : {
        /** Sets up the Mojo and TTO event listeners **/
		setupListeners : function() {
			pageFinalizedFunc = function() {
                TTUApp.functions.hideElementsOnPageFinal();
				TTUApp.pageDoneLoading = true;	
                window.location = 'ttu://pageFinal';
			};
		
			pageLoadingFunc = function() {
				TTUApp.pageDoneLoading = false;
			};
		
			//Page finalized listeners
			Mojo.subscribeForEvent(TTOApp.constants.event.kEvt_FinalizePg, pageFinalizedFunc);
			Mojo.subscribeForEvent(Mojo.constants.events.kPageFinalized, pageFinalizedFunc);
			
			//Page loading listeners
			Mojo.subscribeForEvent(TTOApp.constants.event.kEvt_Submit, pageLoadingFunc);
            
            Mojo.subscribeForEvent(TTOApp.constants.event.kEvt_ModalWindowExists, function(isShown) {
                TTUApp.modalShown = isShown;
                //alert(isShown);
            });
		},
		
        /**Sets up the injected functions for the TTU app **/
		setupInjectedFunctions : function() {
			//Check to see if injected functions has been created
			if(!TTOApp.injectedFunctions) {
				TTOApp.injectedFunctions = {};
			}
			
            //Injected Mojo End flow function.
			TTOApp.injectedFunctions.handleSubmit = function(data) {
                action = '';
				value = '';
                state = TTOApp.Model.getPageInfo().CRstate;
				if(data) {
					if(data.submitData) {
                        action = 'submit';
                        value = data.submitData;
                    }
                    else if(data.jumpData) {
                        action = 'jump';
                        value = data.jumpData.topicPath;
                    }
				}
				redirectURL = 'ttu://endMojo?action=' + action + '&data=' + encodeURIComponent(value) + '&state=' + encodeURIComponent(state);
				window.location = redirectURL;
			};
            
            //Injected App Ready
            TTOApp.injectedFunctions.handleAppReady = function() {
                window.location = 'ttu://appReady';   
            }
		},
        
        windowOpenOverrideFunction : function(url, name, specs, replace) {
            window.location = 'ttu://externalLink?URL=' + encodeURIComponent(url);
        },
        
        windowOnErrorFunction : function(errorMsg, url, lineNumber) {
            console.log(errorMsg + ' line: ' + String(lineNumber));
        },
        
        consoleLogOverrideFunction : function() {
            logString = '';
            for(index in arguments) {
                logString += String(arguments[index]) + ' ';
            }
            window.location = 'ttu://consoleLog?msg=' + encodeURIComponent(logString);
        },
        
        setupOverrideFunctions : function() {
            //Override main windows functions
            window.onerror = this.windowOnErrorFunction;
            window.open = this.windowOpenOverrideFunction;
            console.log = this.consoleLogOverrideFunction;
        },
        
        setupIframeOverrideFunctions : function() {
            iframes = document.getElementsByTagName('iframe');
            
            for(frameIndex in iframes) {
                iframe = iframes[frameIndex];
                if(iframe && typeof(iframe) == 'object') {
                    try {
                        iframe.contentWindow.open = this.windowOpenOverrideFunction;
                    }
                    catch(error){}
                }
            }
        },
		
        /** Basic setup for the webview. **/
		setupWebview : function() {
		
			this.setupListeners();
			//this.setupInjectedFunctions();
            this.setupOverrideFunctions();
            
			ttuMetaTags = document.getElementsByTagName('meta');
			for(metaTagIndex in ttuMetaTags) {
				tag = ttuMetaTags[metaTagIndex];
				if(typeof(tag) == 'object') {
					contentValue = tag.getAttribute('content');
					if(contentValue){
						//contentValue = contentValue.replace('width=device-width', '')
						//tag.setAttribute('content', contentValue);
                        if(contentValue.indexOf('width') > -1) {
                            tag.setAttribute('content', 'initial-scale=1; maximum-scale=1; user-scalable=0;');
                        }
					}
				}
			}
			//document.getElementsByTagName('body')[0].style.overflowX = 'hidden';
		}
    },
    
	functions : {
		/**
		 * Function for iterating through all buttons within the content area
		 * @param buttonAutomationIdOperation function called when a button
		 *  with a valid automationid or id is found. The function should return
		 *  a boolean value to indicate if the button was found. If false the search 
         *  will continue.
		 */
		 runOperationForButtons : function(buttonAutomationIdOperation) {
			buttons = document.getElementById('contentRegion').getElementsByClassName('btn');
			for(buttonIndex in buttons) {
				button = buttons[buttonIndex];
				if(typeof(button) == 'object') {
					automationid = button.getAttribute('automationid');
					buttonId = button.getAttribute('id');
				
					//Check automation id
					if(automationid) {
						automationid = automationid.toLowerCase();
						if(buttonAutomationIdOperation(automationid, button))
							return true;
					}
				
					//Check id
					if(buttonId) {
						buttonId = buttonId.toLowerCase();
						if(buttonAutomationIdOperation(buttonId, button))
							return true;
					}
				}
			}
			return false;
		},
        
        /** Checks if an ID is a member of an id group **/
		doesIdMatchIdKind : function(id, idKinds) {
			for(idKindIndex in idKinds) {
				idKind = idKinds[idKindIndex];
				if(id.indexOf(idKind) > -1)
					return true;
			}
			return false;
		},
        
        /** Hides the navigation buttons on the page **/
		 hideButtons : function() {
			hideButtonFunction = function(id, button) {
				for(i = 0; i < TTUApp.buttonList.length; i++) {
					if(TTUApp.functions.doesIdMatchIdKind(id, TTUApp.buttonList[i])) {
						button.style.display = 'none';
						return false;
					}
				}
			};
		
			this.runOperationForButtons(hideButtonFunction);
		},
		
        /** Hides the element with the given id **/
        hideElementWithID : function(elementID) {
            element = document.getElementById(elementID);
            if(element && typeof(element) == 'object') {
                element.style.setProperty('display', 'none', 'important');
            }
        },
        
        /** Hides elements on the initial set up **/
        hideElementsOnSetup : function() {
            this.hideElementWithID('footerLayer');
            this.hideElementWithID('topbar');
            this.hideElementWithID('main_divider');
            this.hideElementWithID('main_navigation');
            this.hideElementWithID('debug_console_wrapper');
            this.hideElementWithID('debug_console_body');
            this.hideElementsOnPageFinal();
        },
        
        /** Called to hide the elements on page final **/
        hideElementsOnPageFinal : function() {
            this.hideElementWithID('sectionTitle');
            this.hideElementWithID('scroll_fade');
            this.hideButtons();
            TTUApp.setupFunctions.setupIframeOverrideFunctions();
        },
        
        refreshPageWithData : function(data) {
            responseObject = {responseJSON : data};
            TTOApp.Controller.regeneratePage(responseObject);
        },
        
        /** A smoke test that will show an alert **/
        smokeTest : function() {
            alert('Smoke Test');
        }
	}
};
TTUApp.prepareForAppReady();